/****************************************************************************
* DialogBox.java
*
* PURPOSE
* MHealth dialog box.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.lang.*;

class DialogBox extends Dialog {

  private static final int TEXT_WIDTH = 60;

  public DialogBox(Frame parent, String title, String msg) {
    super(parent,title,true);

    GridBagLayout layout = new GridBagLayout();
    setLayout(layout);

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 0;
    gbc.weighty = 0;

    int yOffset=0;
    int index;
    Label lbl;

    while (msg.length() > TEXT_WIDTH) {

      index = msg.indexOf(" ", TEXT_WIDTH);  // find first space after width 

      if (index == -1) 
        break;   // There are no spaces left, break out of loop

      // add line
      lbl = new Label(msg.substring(0, index), Label.CENTER);
      addToGridBag(lbl, gbc, 0, yOffset++, 1, 1);

      // removed portion of msg just displayed
      msg = msg.substring(index);
    }

    // display whatever is left of the msg (last line)
    lbl = new Label(msg, Label.CENTER);
    addToGridBag(lbl, gbc, 0, yOffset++, 1, 1);
    
    Panel p2 = new Panel();
    Button ok = new Button("OK");
    p2.add(ok);
    addToGridBag(p2, gbc, 0, yOffset, 1, 1);

    ok.addActionListener(new ActionListener() { 
                           public void actionPerformed(ActionEvent evt) { 
                             setVisible(false);
                             dispose();
                           } 
                         } );

    addWindowListener(new WindowAdapter() { 
                        public void windowClosing(WindowEvent e) { 
                          setVisible(false);
                          dispose();
                        } 
                      } );

    pack();
    setResizable(false);

    Dimension parent_dim = parent.getSize();
    Point parent_pt = parent.getLocation();
    Dimension dialog_dim = getSize();

    int loc_x = parent_pt.x + ((parent_dim.width - dialog_dim.width) / 2);
    int loc_y = parent_pt.y + ((parent_dim.height - dialog_dim.height) / 2);

    setLocation(new Point(loc_x, loc_y));
  } 

  public void addToGridBag(Component c, GridBagConstraints gbc,
                           int x, int y, int w, int h) {
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    add(c, gbc);
  }
} 
