/****************************************************************************
* Help.java
*
* PURPOSE
* A window of help text.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.io.*;
import java.awt.event.*;

public class Help extends Dialog {

  private MainWindow parent;

  private HelpPanel pnlKey;
  private TextArea txtHelp;
  private Panel pnlButtons;
  private Button btnDismiss;
 
  public Help(MainWindow parent) {
    super(parent, "MHealth Help", true);

    this.parent = parent;

    addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent event) {
                          Dismiss();
                        }
                      });

    setLayout(new BorderLayout());

    pnlKey = new HelpPanel();
    add(pnlKey,"North");

    txtHelp = new TextArea();
    add(txtHelp,"Center");


    pnlButtons = new Panel();

    btnDismiss = new Button("Dismiss");
    btnDismiss.addActionListener(
               new ActionListener() {
                 public void actionPerformed(ActionEvent evt) {
                   Dismiss();
                 } 
               });
    pnlButtons.add(btnDismiss);

    add(pnlButtons,"South");

    addWindowListener(new WindowAdapter() { public void
                          windowClosing(WindowEvent e) { setVisible(false);
                          } } );
  
//    pack();
    int width = 800;
    int height = 600;
    Dimension screensize = getToolkit().getScreenSize();
    if (width > screensize.width)
      width = screensize.width;
    if (height > screensize.height)
      height = screensize.height;
    setSize(width,height);
    setResizable(false);

    // Center the Menu window on the MHealth main window
    Dimension menu_dimension = getSize();
    Dimension parent_dimension = parent.getSize();
    Point parent_point = parent.getLocation();

    setLocation(new Point(parent_point.x +
                        ((parent_dimension.width - menu_dimension.width)/2),
                        parent_point.y +
                        ((parent_dimension.height - menu_dimension.height)/2)));
    

    ReadHelpFile();
    setVisible(true);

  }

  public void Dismiss() {
    setVisible(false);
    dispose();
  }

  public void addToGridBag(Component c, GridBagConstraints gbc,
                           int x, int y, int w, int h) {
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    add(c, gbc);
  }

  public void ReadHelpFile() {
    FindFilePath find = new FindFilePath();
    String path = find.getPath("README.HELP");
    if (path == null) {
      txtHelp.setText("Unable to find README.HELP file.");
      return;
    }
    File f = new File(path, "README.HELP");
    if (!f.exists()) {
      txtHelp.setText("Unable to find README.HELP file.");
      return;
    }
    RandomAccessFile rafile;
    try {
      rafile = new RandomAccessFile(f, "r");
    } catch (IOException ioe) {
      txtHelp.setText("Unable to open README.HELP file.");
      return;
    }
    String s;
    try {
      while ((s = rafile.readLine()) != null) {
        txtHelp.setText(txtHelp.getText() + "\n" + s);
      }
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }
  }

}
