/****************************************************************************
* MHealth.java
*
* PURPOSE
* MHealth main module.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class MHealth implements ActionListener, TimerListener,
                                RtcpLoggerListener, SessionListener {

  static private SessionData sessionData;

  public int receiverCount=0;
  public int receiverTracedCount=0;
  public int sourceCount=0;

  public int imageDumpCountDown=0;

  private static boolean gui=true;  	// gui or non-gui mode

  private FileWriter RouteLog=null;
  private RtcpLogger rtcplogger=null;

  private String CurrentMtraceIP;
  private String CurrentMtraceStatus;
  private ReceiverNode currMtraceReceiverNode = null;

  private Timer timer;

  private Vector sourceReports=null;
  private int selectedSender=0;

  private Mtrace mtrace=null;

  private TreeNode holdparent;

  private ReceiverPath receiverPath;
  
  private MainWindow mainWindow;

  public MHealth() {

    if (gui) {
      mainWindow = new MainWindow(this, sessionData);
    } else { /* not gui */
      sessionData.setRouteLoggingEnabled(true);
      sessionData.setRtcpLoggingEnabled(true);
    }

    timer = new Timer(1000); // fire every second
    timer.addTimerListener(this);

    if ((! sessionData.getSessionIP().equals("")) && 
        (! sessionData.getSessionPort().equals("")) && 
        (sessionData.status == sessionData.STOPPED)) 
        startMHealth();
  }

  public void startMHealth() {
    System.out.println("MHealth: startMHealth()");

    if (gui) {
      mainWindow.setStatusText("Listening for RTCP packets / building sender " 
                               + "and receiver lists...");
      mainWindow.setSessionAddr(sessionData.getSessionIP() + "/" + 
                                sessionData.getSessionPort());
    }

    rtcplogger = new RtcpLogger(sessionData);
    rtcplogger.addRtcpLoggerListener(this);
    rtcplogger.start();

    sessionData.receiverList.setRtcpLogger(rtcplogger);

    if (gui) {
      if (sessionData.tree == null)
        sessionData.tree = new Tree();

      mainWindow.redraw();
    }

//    if (sessionData.getImageDumpEnabled())
//       imageDumpCountDown = Integer.parseInt(sessionData.getImageDumpFreq());

    sessionData.status = sessionData.STARTED;
  }

  public void stopMHealth() {
    System.out.println("MHealth: stopMHealth()");

    if (gui) {
      mainWindow.setStatusText("Stopping MHealth...");
      mainWindow.setSessionAddr("");
    }

    if (rtcplogger != null) {
      System.out.println("MHealth: disabling RtcpLogger");
      rtcplogger.removeRtcpLoggerListener(this);
      rtcplogger.die();
      rtcplogger = null;
    }


    System.out.println("MHealth: disabling mtrace");
    stopMtrace(); // must occur BEFORE timer stop!

/*    if (timer != null) {
      if (DEBUG) System.out.println("disabling timer");
      timer.removeTimerListener();
      timer = null;
    }*/

    sourceReports = null;

    if (sessionData.receiverList != null) {
      sessionData.receiverList.delete();
      sessionData.receiverList=null;
    }
    if (sessionData.sourceList != null) {
      sessionData.sourceList.delete();
      sessionData.sourceList=null;
    }

    if (gui) {
      if (sessionData.tree != null) {
        sessionData.tree.delete();
        sessionData.tree=null;
      }

      mainWindow.redraw();
    }

    sourceCount=0;
    receiverCount=0;
    receiverTracedCount=0;

    if (gui) {
      mainWindow.setSourceCount(sourceCount + "");
      mainWindow.setReceiverCount(receiverTracedCount + "/" + receiverCount);
    }
    sessionData.status = sessionData.STOPPED;

  }

  public void actionPerformed(ActionEvent e) {
    Object origin;

    origin = e.getSource();

    if (origin == mainWindow.btnQuit) {
      gracefulShutdown();
    }
  }

  public void senderAdded(RtcpReport report) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: senderAdded(" + 
//              report.getIP() + ")");

    sourceCount++;

    sessionData.sourceList.addSourceNode(report);

    if (gui) { 
      mainWindow.setSourceCount(sourceCount + "");

      if (sessionData.tree.getRoot() == null) {
        TreeNode tn = new TreeNode(report.getName(),report.getIP());
        sessionData.tree.setRoot(tn);
      }
      mainWindow.redraw();
    }
  }

  public void senderRemoved(RtcpReport report) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: senderRemoved(" + 
//              report.getIP() + ")");

      sourceCount--;
   
    if (gui) {
      mainWindow.setSourceCount(sourceCount + "");
    }
  }

  public void senderUpdated(RtcpReport report, boolean redraw) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: senderUpdated(" + 
//              report.getIP() + ")");

    if (redraw) mainWindow.redraw();
  }

  public void receiverAdded(RtcpReport report) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: receiverAdded(" + 
//              report.getIP() + ")");

    receiverCount++;

    sessionData.receiverList.addReceiverNode(report);

    if (gui) {
      mainWindow.setReceiverCount(receiverTracedCount + "/" + receiverCount);

      mainWindow.redraw();
    }
  }

  public void receiverRemoved(RtcpReport report) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: receiverRemoved(" + 
//              report.getIP() + ")");

    receiverCount--;

    sessionData.receiverList.removeReceiverNode(report);

    if (removeReceiver(report.getIP())) 
      receiverTracedCount--;

    if (gui) {
      mainWindow.setStatusText("Removing receiver " + report.getName() + " / " +
                               report.getIP());

      mainWindow.setReceiverCount(receiverTracedCount + "/" + receiverCount);
      mainWindow.redraw();
    }
  }

  public void receiverTimeout(RtcpReport report) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: receiverTimeout(" + 
//              report.getIP() + ")");

    sessionData.receiverList.timeoutReceiverInList(report.getIP());
    if (gui)
      mainWindow.redraw();
  }

  public void receiverUpdated(RtcpReport report, boolean redraw) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: receiverUpdated(" + 
//              report.getIP() + ")");

    if (redraw) mainWindow.redraw();
  }

  public void timeElapsed(TimerEvent evt) {
//    System.out.println("timeElapsed");
    if (gui)
      mainWindow.setLocalTime(new Date().toString());

    if (mtrace == null)
       startMtrace();


    if (sessionData.getImageDumpEnabled()) {
      imageDumpCountDown--;
      if (imageDumpCountDown < 0) {
        if (gui) {
          mainWindow.setTempStatusText("Generating image dump...");
          mainWindow.createImageDump();
          mainWindow.unsetTempStatusText();
        }
        imageDumpCountDown = Integer.parseInt(sessionData.getImageDumpFreq());
      }
    }
  }

  private boolean startMtrace() {
//    System.out.println("MHealth: startMtrace()");

    if (mtrace != null) {
//      System.out.println("MHealth: startMtrace() failed: mtrace in progress");
      return(false); // mtrace already in progress
    }
    
    if (sessionData.sourceList.isEmpty()) {
//      System.out.println("MHealth: startMtrace() failed: no source");
      return(false); // nothing to trace yet!
    }

    if (sessionData.receiverList.isEmpty()) {
//      System.out.println("MHealth: startMtrace() failed: no receivers");
      return(false); // nothing to trace yet!
    }

    currMtraceReceiverNode = sessionData.receiverList.getReceiverToMtrace();

    if (currMtraceReceiverNode == null) {
//      System.out.println("MHealth: startMtrace().getReceiverToMtrace() null");
      return(false); // nothing to trace yet!
    }

    CurrentMtraceStatus = "Mtracing";
    CurrentMtraceIP = currMtraceReceiverNode.getIP();

    // we don't know if box is in receiverList or tree, so set both:
    sessionData.receiverList.setMessage(CurrentMtraceIP,"Mtracing...");

    if (gui) {
      sessionData.tree.setMessage(CurrentMtraceIP,"Mtracing...");
      mainWindow.redraw();

      mainWindow.setStatusText("Mtracing receiver: " + 
                               currMtraceReceiverNode.getName() + " / "
                               + currMtraceReceiverNode.getIP());
    }

    mtrace = new Mtrace(this, sessionData, currMtraceReceiverNode);
    mtrace.start();

    return(true);
  }

  private void stopMtrace() {
    if (mtrace != null) {
      mtrace.killMtrace();
      mtrace = null;
    }
  }

  private void stopMtraceAndReset() {
    stopMtrace();
//    startTimer();
  }

/*  private void startTimer() {
    timer = new Timer(delay);
    timer.addTimerListener(this);
  }*/

  public void mtraceDone(ReceiverPath receiverPath) {

    this.receiverPath = receiverPath;

    if (receiverPath.indexLastHop > 0) {
      if ((receiverPath.mtraceHop[receiverPath.indexLastHop].getIP().equals(
              sessionData.sourceList.getSourceRoot().getIP())) &&
          (receiverPath.mtraceHop[0].getIP().equals(CurrentMtraceIP))) {
        if (gui) {
          if (! sessionData.tree.setMessage(CurrentMtraceIP,"Mtrace: COMPLETE")) 
            receiverTracedCount++;
          addPath();
        }

        // we don't know if box is in receiverList or tree, so set both:
        sessionData.receiverList.setMessage(CurrentMtraceIP,"Mtrace: COMPLETE");

        if (gui) 
          sessionData.tree.setMessage(CurrentMtraceIP,"Mtrace: COMPLETE");

        CurrentMtraceStatus = "COMPLETE";
        if (gui) {
          mainWindow.setStatusText("Mtrace successful.");
          mainWindow.setReceiverCount(receiverTracedCount + "/" 
                                      + receiverCount);
        }
      } else {
        // we don't know if box is in receiverList or tree, so set both:
        sessionData.receiverList.setMessage(CurrentMtraceIP,"Mtrace: FAILED");
        if (gui) {
          sessionData.tree.setMessage(CurrentMtraceIP,"Mtrace: FAILED");
        }
        
        CurrentMtraceStatus = "FAILED";

        if (gui) 
          mainWindow.setStatusText("Mtrace failed.");
      }
    } else {
      // we don't know if box is in receiverList or tree, so set both:
      sessionData.receiverList.setMessage(CurrentMtraceIP,"Mtrace: FAILED");
      if (gui) {
        sessionData.tree.setMessage(CurrentMtraceIP,"Mtrace: FAILED");
      }

      CurrentMtraceStatus = "FAILED";
  
      if (gui) 
        mainWindow.setStatusText("Mtrace failed.");
    }
    if (gui) {
      for (int x=0; x < receiverPath.indexLastHop; x++) {
        if (receiverPath.mtraceHop[x].getLostPackets() != null)
          sessionData.tree.setMtraceLoss(receiverPath.mtraceHop[x].getIP(), 
                             receiverPath.mtraceHop[x].getLostPackets(), 
                             receiverPath.mtraceHop[x].getTotalPackets());
      }
      mainWindow.redraw();
    }
    stopMtraceAndReset();
  }

  public void addPath() {
    boolean root = true;
    TreeNode parent, tn;
    int y;
    String holdip = new String("");

//    System.out.println("MHealth: addPath(" + receiverPath.indexLastHop + ")");
    
    sessionData.tree.addPath(receiverPath,currMtraceReceiverNode);
/*  
    for (int x=receiverPath.indexLastHop-1; x>=0; x--) {
//      System.out.println("new TreeNode(" + receiverPath.mtraceHop[x].getName()
//                         + "," + mtraceHop[x].getIP() +")");
      if (! receiverPath.mtraceHop[x].getIP().equals(holdip)) {
        tn = new TreeNode(receiverPath.mtraceHop[x].getName(),
                          receiverPath.mtraceHop[x].getIP());
        tn.mtraceStats = new MtraceStats(CurrentMtraceIP, 
                                         receiverPath.mtraceStart, 
                                         receiverPath.mtraceFinish);
        if (receiverPath.mtraceHop[x].getIP().equals(CurrentMtraceIP)) {
          tn.setRtcpReport(currMtraceReceiverNode.getRtcpReport());
        }
        if (root == true) {
          parent = sessionData.tree.getRoot();
          root = false;
        } else {
          if (holdparent.IP == null) 
            System.out.println("MHealth: ERROR: addPath() " +
                      "Warning: holdparent = null at 2");
          if (sessionData.tree.getRoot() == null) 
            System.out.println("MHealth: ERROR: addPath() " +
                      "Warning: sessionData.tree.getRoot() = null at 2");
          //parent = sessionData.tree.findNode(sessionData.tree.getRoot(),holdparent);
        }

        //holdparent = sessionData.tree.addNode(tn, parent);
  
        holdip = new String(receiverPath.mtraceHop[x].getIP());
      }
    }*/
    mainWindow.redraw();
  }

  public String getCurrentMtraceIP() { return(CurrentMtraceIP); }

  public String getCurrentMtraceStatus() { return(CurrentMtraceStatus); }

  public void timeoutReceiver(String IP) {
    if (gui) {
      if (sessionData.tree != null) 
        sessionData.tree.timeoutReceiverInTree(IP);
    }

    sessionData.receiverList.timeoutReceiverInList(IP);

    if (gui) 
      mainWindow.redraw();
  }

  public boolean removeReceiver(String IP) {
//    Debug.out(debugLevel, Debug.INFO, "MHealth: removeReceiver(" + IP + ")");
//    Debug.out(debugLevel, Debug.INFO, "MHealth: CurrentMtraceIP =" + 
//                                                CurrentMtraceIP );
    boolean ret = false;
    if (sessionData.tree != null) {
      ret = sessionData.tree.removeReceiverFromTree(IP);
    }
    if (IP.equals(CurrentMtraceIP)) {
      stopMtraceAndReset();
      sessionData.tree.setMessage(CurrentMtraceIP, "Mtrace: CANCELLED");
      mainWindow.redraw();
    }
    return(ret);
  }

  public void gracefulShutdown() {
    stopMHealth();
    
    System.exit(0);
  }

  public Tree getTree() { return(sessionData.tree); }

/*************************************************************************
 ***************** implements interface SessionListener ******************
 *************************************************************************/
  public void addressChanged(String MCIP, String MCport) {
    stopMHealth();
    startMHealth();
  }

  public void logFileNameDirChanged() {
    if (rtcplogger != null) 
       rtcplogger.logFileNameDirChanged();
    if (mtrace != null) 
       mtrace.logFileNameDirChanged();
  }

  public void rtcpLoggingChanged(boolean rtcpLogging) {
//    System.out.println("rtcpLoggingChanged(" + rtcpLogging + ")");
    if (rtcplogger != null) {
      if (rtcpLogging)
        rtcplogger.startLogging();
      else
        rtcplogger.stopLogging();
    }
  }

  public void routeLoggingChanged(boolean routeLogging) {
    if (mtrace != null) {
      if (routeLogging)
        mtrace.startLogging();
      else
        mtrace.stopLogging();
    }
  }

/*************************************************************************
 ********************* end of interface SessionListener ******************
 *************************************************************************/

  public static boolean parseAddr(String addr) { 

    // expected: string in form of "IP/port"

    int holdloc = addr.indexOf("/");
    if (holdloc == -1) return(false);
    sessionData.setSessionIP(new String(addr.substring(0,holdloc)));
    sessionData.setSessionPort(new String(addr.substring(holdloc+1)));

    ValidAddress validaddress = new ValidAddress();
    return(validaddress.isValidMulticastAddressPort(sessionData.getSessionIP(),
           sessionData.getSessionPort()));
  }

  public static void Usage() {
    System.out.print("Usage: MHealth ");
    System.out.print("[multicastIP/multicastPort]");
    System.out.println("[-nogui] [-o logfilename] [-d logfiledir]");
    System.exit(0);
  }

  public static void main(String argv[]) {

    sessionData = new SessionData();

    for (int x=0; x < argv.length; x++) {
      if (argv[x].indexOf("/") != -1) {
        /* this should be the address/port */

        if (! parseAddr(argv[x])) {
          System.out.println("Error: unable to parse ip/port " + 
                             "or invalid ip/port.");
          Usage();
        }
      } else if (argv[x].equals("-nogui")) {
        System.out.println("nogui");
        gui = false;
        continue;
      } else if (argv[x].equals("-o")) {
        x++;
        if (x > argv.length) {
          System.out.println("-o option requires a filename argument.");
          Usage();
        }
        sessionData.setLogFileName(argv[x]);
        System.out.println("LogFileName will be " + argv[x]);
        continue;
      } else if (argv[x].equals("-d")) {
        x++;
        if (x > argv.length) {
          System.out.println("-d option requires a directory argument.");
          Usage();
        }
        File f = new File(argv[x]);
        if (!f.isDirectory()) {
          System.out.println("-d Directory is not valid.");
          Usage();
        }
        sessionData.setLogDir(argv[x]);
        continue;
      } else
        Usage();
    }

    MHealth m = new MHealth();
  }
}
