/****************************************************************************
* MainWindow.java
*
* PURPOSE
* Primary display window of MHealth.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class MainWindow extends Frame implements ActionListener {

  private MHealth parent;
  private SessionData sessionData;

  private Menu menu;

  // Graphical components:

  private ScrollableTreePanel treepanel = null;

  // Bottom panel, primary buttons
  private Panel bottomPanel;
  public Button btnMenu, btnPrint, btnHelp, btnQuit;

  // Top panel, session, status, and other info
  private BorderedPanel topPanel;
  private TextField txtSessionAddr;
  private TextField txtLocalTime;
  private TextField txtSourceCount;
  private TextField txtReceiverCount;
  private TextField txtStatus;

  // Left and Right panels, for future use
  private Panel leftPanel;
  private Panel rightPanel;

  public MainWindow(MHealth parent, SessionData sessionData) {
    super("MHealth v1.0");

    this.parent = parent;
    this.sessionData = sessionData;

    Dimension screensize = getToolkit().getScreenSize();
    setSize(screensize.width-100,screensize.height-100);
    //setSize(1000,820);
    setLocation(50,10);

    // call gracefulShutdown when window box is closed
    addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent event) {
                    //      parent.gracefulShutdown();
                        }
                      });

    setLayout(new BorderLayout());

    // Center is the tree panel
    treepanel = new ScrollableTreePanel(sessionData, 900, 700);
    add(treepanel, "Center");

    // Top Panel of controls
    topPanel = new BorderedPanel();
    topPanel.setWeight(1,1);

    // MHealth session address
    topPanel.addToGridBag(new Label("Session Address: ",Label.RIGHT), 
                          0, 0, 1, 1);
    txtSessionAddr = new TextField("",20);
    txtSessionAddr.setEditable(false);
    topPanel.addToGridBag(txtSessionAddr, 1, 0, 1, 1);

    // local time/date
    topPanel.addToGridBag(new Label("Local Time: ",Label.RIGHT), 
                          2, 0, 1, 1);
    txtLocalTime = new TextField(new Date().toString(),40);
    txtLocalTime.setEditable(false);
    topPanel.addToGridBag(txtLocalTime, 3, 0, 6, 1);

    // Source Count
    topPanel.addToGridBag(new Label("Source Count: ",Label.RIGHT), 
                          0, 2, 1, 1);
    txtSourceCount = new TextField("0",40);
    txtSourceCount.setEditable(false);
    topPanel.addToGridBag(txtSourceCount, 1, 2, 1, 1);

    // Receiver Count
    topPanel.addToGridBag(new Label("Receivers (traced/total): ",
                          Label.RIGHT), 2, 2, 1, 1);
    txtReceiverCount = new TextField("0/0",40);
    txtReceiverCount.setEditable(false);
    topPanel.addToGridBag(txtReceiverCount, 3, 2, 6, 1);

    // MHealth status bar
    topPanel.addToGridBag(new Label("Status: ", Label.RIGHT), 0, 3, 1, 1);
    txtStatus = new TextField("",80);
    txtStatus.setEditable(false);
    topPanel.addToGridBag(txtStatus, 1, 3, 10, 1);

    add(topPanel,"North");

    // Left Panel of controls (for future use)
    leftPanel = new Panel(new FlowLayout());
    add(leftPanel,"West");

    // Right Panel of controls (for future use)
    rightPanel = new Panel(new FlowLayout());
    add(rightPanel,"East");

    // Bottom Panel of buttons
    bottomPanel = new Panel(new FlowLayout());

    btnMenu = new Button("Menu");
    btnMenu.addActionListener(this);
    bottomPanel.add(btnMenu);

    btnPrint = new Button("Print");
    btnPrint.addActionListener(this);
    bottomPanel.add(btnPrint);

    btnHelp = new Button("Help");
    btnHelp.addActionListener(this);
    bottomPanel.add(btnHelp);

    btnQuit = new Button("Quit");
    btnQuit.addActionListener(parent);
    bottomPanel.add(btnQuit);

    add(bottomPanel,"South");

    setVisible(true);

    // default to the Menu window on startup
    showMenu();
  }

  public void showMenu() {

    // display modal dialog window of menu option for MHealth

    menu = new Menu(this, parent, sessionData);
    menu.addSessionListener(parent);

    // cleanup menu frame
    menu.dispose();
    menu = null;
  }

  public String getStatusText() { return(txtStatus.getText()); }
  public void setStatusText(String text) {
    txtStatus.setText(text);
    sessionData.setStatusText(text);
  }
  public void setTempStatusText(String text) {
    // used to display "generating image dump" message on screen but
    // output a meanful message to the html file.
    txtStatus.setText(text);
  }
  public void unsetTempStatusText() {
    // used to revert "generating image dump" message to prior message
    txtStatus.setText(sessionData.getStatusText());
  }

  public String getSessionAddr() { return(txtSessionAddr.getText()); }
  public void setSessionAddr(String text) {
    txtSessionAddr.setText(text);
    sessionData.setSessionAddr(text);
  }

  public void setSourceCount(String text) {
    txtSourceCount.setText(text);
    sessionData.setSourceCount(text);
  }

  public void setReceiverCount(String text) {
    txtReceiverCount.setText(text);
    sessionData.setReceiverCount(text);
  }

  public void setTitle(String text) {
    super.setTitle(text);
  }

  public void setLocalTime(String text) {
    txtLocalTime.setText(text);
    sessionData.setLocalTime(text);
  }

  public void actionPerformed(ActionEvent e) {
    Object origin;

    origin = e.getSource();

    if (origin == btnMenu) {
      showMenu();
    }
    if (origin == btnHelp) {
      Help h = new Help(this);
    }
    if (origin == btnPrint) {
      PrintJob pjob = getToolkit().getPrintJob(this, "MHealth", null);
      if (pjob != null) {
        Graphics pg = pjob.getGraphics();
        if (pg != null) {
          treepanel.printAll(pg);
          pg.dispose();
        }
        pjob.end();
      }
    }
  }

  public void addToGridBag(GridBagLayout gbl, Component p, Component c, 
                           GridBagConstraints gbc, int x, int y, int w, int h) {
      gbc.gridx = x;
      gbc.gridy = y;
      gbc.gridwidth = w;
      gbc.gridheight = h;
      gbl.setConstraints(c,gbc);
      add(c);
  }

  public void redraw() { 
//    System.out.println("MainWindow: redraw()");
    if (treepanel != null)
      treepanel.redraw();
  }

  public void createImageDump() {
    treepanel.createImageDump();
  }

}
