/****************************************************************************
* MtraceStats.java
*
* PURPOSE
* Class to hold mtrace statistics.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.util.*;
import java.awt.*;
import java.io.*;

public class MtraceStats {

  public final static int STATUS_FAIL    = -1;
  public final static int STATUS_SUCCESS =  1;

  private Date mtraceStartDate=null;
  private Date mtraceFinishDate=null;

  private String mtraceReceiver="";
  private int mtraceStatus=0;

  private String mtraceNodeFailed=null;
  private String mtraceReasonFailed=null;

  private String strLoss = "";
  private int intLostPackets = -1;
  private int intTotalPackets = -1;
  private String strLossPercent = "?";
  private int intLossPercent = 0;

  private int roundTripTime;

  public MtraceStats(String receiver, Date start, Date finish) {
    mtraceReceiver = receiver; 
    mtraceStartDate = start; 
    mtraceFinishDate = finish; 
    mtraceStatus = STATUS_SUCCESS; 
  }

  public Date getMtraceStartDate() { return(mtraceStartDate); }

  public Date getMtraceFinishDate() { return(mtraceFinishDate); }

  public String getMtraceReceiver() { return(mtraceReceiver); } 

  public String getMtraceStatus() { 
    switch(mtraceStatus) {
    case STATUS_SUCCESS:
      return("Successful");
    case STATUS_FAIL:
      return("Failed");
    default:
      return("Unknown");
    }
  }

  public String getMtraceNodeFailed() { return(mtraceNodeFailed); }

  public String getMtraceReasonFailed() { return(mtraceReasonFailed); }

  public void setMtraceFailed(String node, String reason) {
    mtraceStatus = STATUS_FAIL;
    mtraceNodeFailed = node; 
    mtraceReasonFailed = reason; 
  }

  public int getMtraceLostPackets() { return(intLostPackets); }

  public int getMtraceTotalPackets() { return(intTotalPackets); }

  public String getMtraceLoss() { return(strLoss); }

  public int getMtraceLossPercent() { return(intLossPercent); }

  public String getMtraceLossPercentStr() { return(strLossPercent); }

  public void setMtraceLoss(String lost, String total) {
    if ((lost.equals("")) || (total.equals("")))
      strLoss = "";
    else
      strLoss = lost + "/" + total;

    try {
      intLostPackets = Integer.parseInt(lost);
    } catch (NumberFormatException nfe) {
      intLostPackets = -9999;
    }

    try {
      intTotalPackets = Integer.parseInt(total);
    } catch (NumberFormatException nfe) {
      intTotalPackets = -9999;
    }

    if ((intTotalPackets == -9999) || (intLostPackets == -9999)) {
      intLossPercent = -1;
      strLossPercent = "?";
    } else {
      if (intLostPackets <= 0) 
        intLossPercent = 0;
      else
        intLossPercent = (int)(100.0 * (float)intLostPackets 
                               / (float)intTotalPackets);
      strLossPercent = intLossPercent + "%";
    }
  }
}
