/****************************************************************************
* PopupMenuEvent.java
*
* PURPOSE
* Custom event for clicking on nodes to get menus.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;

public class PopupMenuEvent implements MouseListener, ActionListener {

  private PopupMenu popup;

  private TreePanel treepanel = null;
  private ScrollInterface scroller = null;

  private Tree tree = null;
  private ReceiverList receiverList = null;
  private SourceList sourceList = null;

  private TreeNode treenode = null;
  private ReceiverNode receivernode = null;
  private SourceNode sourcenode = null;

  private int popupX, popupY;
  private int XVal, YVal;

  private MenuItem itemPrune;
  private MenuItem itemExpand;
  private MenuItem itemStats;
  private MenuItem itemNext;
  private MenuItem itemMakeRoot;
  private MenuItem itemDebug;
 
  public PopupMenuEvent(TreePanel treepanel, ScrollInterface scroller, 
                        Tree tree, ReceiverList receiverList, 
                        SourceList sourceList) {
    //Debug.out(debugLevel, Debug.INFO, "new PopupMenuEvent()");

    //Debug.out(debugLevel, Debug.ERROR, "PopupMenuEvent: tree is null");

    this.treepanel = treepanel;
    this.scroller = scroller;
    this.tree = tree;
    this.sourceList = sourceList;
    this.receiverList = receiverList;

    treepanel.addMouseListener(this);
  }

  public void processMouseEvent(MouseEvent evt) {
    //Debug.out(debugLevel, Debug.INFO, "PopupMenuEvent: processMouseEvent()");

     switch (evt.getID()) {
     case MouseEvent.MOUSE_PRESSED:
       showPopup(evt);
       break;
     case MouseEvent.MOUSE_RELEASED:
       break;
     }
  }

  public void mouseClicked(MouseEvent evt) {
  }

  public void mousePressed(MouseEvent evt) {
    //Debug.out(debugLevel, Debug.INFO, "PopupMenuEvent: mousePressed()");

    XVal = evt.getX();
    YVal = evt.getY();
    //Debug.out(debugLevel, Debug.DATA, "PopupMenuEvent: XVal=" + XVal + 
    //          ", YVal=" + YVal);

    if (scroller != null) {
      //Debug.out(debugLevel, Debug.DATA, "PopupMenuEvent: horiz val="
      //          + scroller.getHorizontalScrollerVal() + ",verti val=" + 
      //          scroller.getVerticalScrollerVal());
      XVal += scroller.getHorizontalScrollerVal();
      YVal += scroller.getVerticalScrollerVal();
      //Debug.out(debugLevel, Debug.DATA, "PopupMenuEvent: XVal=" + XVal + 
      //          ", YVal=" + YVal);
    }

    showPopup(evt);
  }

  public void mouseReleased(MouseEvent evt) { } 
  public void mouseEntered(MouseEvent evt) { }
  public void mouseExited(MouseEvent evt) { }
  public void mouseDragged(MouseEvent evt) { }
  public void mouseMoved(MouseEvent evt) { }

  public void showPopup(MouseEvent evt) {
    treenode = null;
    receivernode = null;
    sourcenode = null;

    if (receiverList != null) { 
      receivernode = receiverList.findReceiverInside
                                    (receiverList.getReceiverRoot(),XVal,YVal);

      if (receivernode != null) {
        addReceiverNodeMenu(evt);
      }
    }
    if (tree != null) {
      treenode = tree.findInside(tree.getRoot(), XVal, YVal); 

      if (treenode != null) {
        addTreeNodeMenu(evt);
        return;
      }
    }
    if (sourceList != null) { 
      sourcenode = sourceList.findSourceInside
                                    (sourceList.getSourceRoot(),XVal,YVal);

      if (sourcenode != null) {
        addSourceNodeMenu(evt);
      }
    }
  }
  
  public void addTreeNodeMenu(MouseEvent evt) {
    popupX = evt.getX();
    popupY = evt.getY();

    popup = new PopupMenu();
    popup.add(new MenuItem(treenode.name + "/" + treenode.IP));
    popup.addSeparator();
    if (treenode.hasPrunedChildren()) {
      popup.add(itemExpand = new MenuItem("Expand"));
      itemExpand.addActionListener(this);
    } else {
      if (! treenode.isRoot()) {
        popup.add(itemPrune = new MenuItem("Prune"));
        itemPrune.addActionListener(this);
      } 
    }
    if (treenode.isReceiver()) {
       popup.add(itemNext = new MenuItem("Mtrace Next"));
       itemNext.addActionListener(this);
    }
    popup.add(itemStats = new MenuItem("View Stats"));
    itemStats.addActionListener(this);
//    popup.add(itemDebug = new MenuItem("Debug Info"));
//    itemDebug.addActionListener(this);

    treepanel.add(popup);

    popup.show(treepanel, popupX, popupY);
  }

  public void addReceiverNodeMenu(MouseEvent evt) {
    popupX = evt.getX();
    popupY = evt.getY();

    popup = new PopupMenu();

    popup.add(new MenuItem(receivernode.getName() + "/" + 
                           receivernode.getIP()));
    popup.addSeparator();

    popup.add(itemNext = new MenuItem("Mtrace Next"));
    itemNext.addActionListener(this);

    popup.add(itemStats = new MenuItem("View Stats"));
    itemStats.addActionListener(this);
 
    treepanel.add(popup);

    popup.show(treepanel, popupX, popupY);

  }
  
  public void addSourceNodeMenu(MouseEvent evt) {
    popupX = evt.getX();
    popupY = evt.getY();

    popup = new PopupMenu();

    popup.add(new MenuItem(sourcenode.getName() + "/" + 
                           sourcenode.getIP()));
    popup.addSeparator();

    popup.add(itemStats = new MenuItem("View Stats"));
    itemStats.addActionListener(this);
 
    popup.add(itemMakeRoot = new MenuItem("Make Root"));
    itemMakeRoot.addActionListener(this);

    treepanel.add(popup);

    popup.show(treepanel, popupX, popupY);
  }

  public void actionPerformed(ActionEvent e) {
    String command = e.getActionCommand();

    if ((treenode == null) && (receivernode == null) && (sourcenode == null)) 
      return;

    if (command.equals("Prune")) {
      tree.pruneNode(treenode); 
      treepanel.redraw();
    }
    if (command.equals("Expand")) {
      tree.expandChildren(treenode); 
      treepanel.redraw();
    }
    if (command.equals("Mtrace Next")) {
      if (receivernode != null) {
        receiverList.setNextMtrace(receivernode.getIP());
        receivernode.setMessage("Mtrace: SCHEDULED");
        treepanel.redraw();
      } 
      if (treenode != null) {
        receiverList.setNextMtrace(treenode.IP);
        treenode.setMessage("Mtrace: SCHEDULED");
        treepanel.redraw();
      } 
    }
    if (command.equals("Debug Info")) {
      if (treenode != null) 
        treenode.debugDump();
    }
    if (command.equals("View Stats")) {
      if (treenode != null) {
        if (treenode.isRoot()) {
          StatsWin statsWin = new StatsWin(treenode, 
                                   sourceList.getSourceRoot().getRtcpReport(), 
                                   receiverList.getRtcpLogger(), 
                                   popupX, popupY);
          statsWin.show();
        } else if (treenode.isReceiver()) {
          StatsWin statsWin = new StatsWin(treenode, 
                                           treenode.getRtcpReport(), 
                                           receiverList.getRtcpLogger(), 
                                           popupX, popupY);
          statsWin.show();
        } else {
          StatsWin statsWin = new StatsWin(treenode, 
                                           null,
                                           receiverList.getRtcpLogger(), 
                                           popupX, popupY);
          statsWin.show();
        }
      } else if (receivernode != null) {
        // this is an unplaced receiver
        StatsWin statsWin = new StatsWin(null,
                                   receivernode.getRtcpReport(),
                                   receiverList.getRtcpLogger(),
                                   popupX, popupY-100);
        statsWin.show();
      } else if (sourcenode != null) {
        // this is an source node that is not root
        StatsWin statsWin = new StatsWin(null,
                                   sourcenode.getRtcpReport(),
                                   receiverList.getRtcpLogger(),
                                   popupX, popupY);
        statsWin.show();
      }
    }
    if (command.equals("Make Root")) {
        //Frame f = new Frame();
        //f.setSize(100,100);
        DialogBox db = new DialogBox(new Frame(), "Function not implemented!",
          "Sorry, this function has not been implemented yet!");
        db.show();
    }
  }

}
