/****************************************************************************
* ReceiverNode.java
*
* PURPOSE
* Class to hold info on a single multicast receiver.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.io.*;

public class ReceiverNode {

  private String message;

  private ReceiverNode nextNode = null;
  private ReceiverNode prevNode = null;

  private boolean placedInTree = false;
  private boolean timeout = false;

  private RtcpReport report;
  
  private int x=0;
  private int y=0;
  private Dimension dimensions;

  private String lastHopIP=null;

  // some box drawing color constants
  private Color receiver_box_color  = Color.green;
  private Color timeout_box_color   = Color.lightGray;
  private Color box_outline_color   = Color.black;
  private Color warning_color       = Color.yellow;
  private Color congested_color     = Color.red;
  private Color no_data_color       = Color.pink;
  private Color box_text_color      = Color.black;
  private Color msg_text_color      = Color.black;
      
  public ReceiverNode(RtcpReport report) {
    this.report=report;
  }

  public boolean hasNext() {
    return(nextNode != null);
  }

  public boolean isTail() {
    return(nextNode == null);
  }

  public void setCoordinates(int x, int y, int width, int height) {
    this.x = x;
    this.y = y;
    dimensions = new Dimension(width, height);
  }

  public int getX() { return(x); }
  public void setX(int val) { x = val; }

  public int getY() { return(y); }
  public void setY(int val) { y = val; }

  public Dimension getDimensions() { return(dimensions); }
  public void setDimensions(Dimension val) { dimensions = val; }
  public void setDimensions(int width, int height) { 
    dimensions = new Dimension(width,height); 
  }

  public boolean getTimeout() { return(timeout); }
  public void setTimeout(boolean Val) { timeout = Val; }

  public boolean isInside(int mouseX, int mouseY) {
    if (placedInTree) return(false);

    if ((mouseX > this.x) && (mouseX < (this.x + dimensions.width)) &&
        (mouseY > this.y) && (mouseY < (this.y + dimensions.height)))
      return(true);
    else
      return(false);
  }

  public void draw(Graphics g) {
    FontMetrics fm;
    int font_width;
    int fraction_lost;

    if (getTimeout())
      g.setColor(timeout_box_color);
    else if (report.getReportCount() == 0)
      g.setColor(no_data_color);
    else if (report.report_blocks[0].fraction_lost > 23) 
      g.setColor(congested_color);
    else if (report.report_blocks[0].fraction_lost > 12) 
      g.setColor(warning_color);
    else
      g.setColor(receiver_box_color);
    g.fillRect(x, y, dimensions.width, dimensions.height);

    g.setColor(box_outline_color);
    g.drawRect(x, y, dimensions.width, dimensions.height);

    g.setColor(box_text_color);
    fm = g.getFontMetrics(g.getFont());
    font_width = fm.stringWidth(report.getName());
    if (fm.stringWidth(report.getName()) > dimensions.width) {
      // name doesn't fit in box, let's try IP address
      font_width = fm.stringWidth(report.getIP());
      g.drawString (report.getIP(), x + ((dimensions.width-font_width)/2), 
                    y+15);
    } else {
      font_width = fm.stringWidth(report.getName());
      g.drawString (report.getName(), x + ((dimensions.width-font_width)/2), 
                    y+15);
    }

    if (message != null) {
      g.setColor(msg_text_color);
      font_width = fm.stringWidth(message);
      g.drawString (message, x + ((dimensions.width-font_width)/2), y+30);
    }

  }

  public RtcpReport getRtcpReport() { return(report); }

  public String getName() { return(report.getName()); }

  public String getIP() { return(report.getIP()); }

  public void setMessage(String message) { this.message = message; } 
  public String getMessage() { return(message); }

  public void setNextNode(ReceiverNode nextNode) { this.nextNode = nextNode; }
  public ReceiverNode getNextNode() { return(nextNode); }

  public void setPrevNode(ReceiverNode prevNode) { this.prevNode = prevNode; }
  public ReceiverNode getPrevNode() { return(prevNode); }

  public void setPlacedInTree(boolean val) { placedInTree = val; }
  public boolean getPlacedInTree() { return(placedInTree); }

  public void setLastHopIP(String IP) { lastHopIP = IP; }
  public String getLastHopIP() { return(lastHopIP); }

}
