/****************************************************************************
* ReceiverPath.java
*
* PURPOSE
* Class to hold info on a single multicast receiver's path to the source.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.util.*;

public class ReceiverPath {

  private String receiverName;
  private String receiverIP;

  public MtraceHop mtraceHop[];
  public int indexLastHop=0;

  public Date mtraceStart, mtraceFinish;

  public ReceiverPath(String receiverName, String receiverIP) {
    this.receiverName   = receiverName;
    this.receiverIP     = receiverIP;
  }

  public String getReceiverName() { return(receiverName); }
  public void setReceiverName(String receiverName) { 
    this.receiverName = receiverName; 
  }

  public String getReceiverIP() { return(receiverIP); }
  public void setReceiverIP(String receiverIP) { 
    this.receiverIP = receiverIP; 
  }

  public boolean isSamePath(ReceiverPath receiverPath) {
    if (this.indexLastHop != receiverPath.indexLastHop) 
      return(false);
   
    for (int x=0; x < this.indexLastHop; x++) {
      if (!this.mtraceHop[x].getIP().equals(receiverPath.mtraceHop[x].getIP()))
        return(false); 
    }
    return(true); 
  }

  public boolean update(ReceiverPath newPath) {
    if (! this.getReceiverIP().equals(newPath.getReceiverIP())) 
       return(false);
    this.indexLastHop = newPath.indexLastHop;
    this.mtraceHop = newPath.mtraceHop;
    this.mtraceStart = newPath.mtraceStart;
    this.mtraceFinish = newPath.mtraceFinish;
    return(true);
  }
}
