/****************************************************************************
* RtcpReportHeader.java
*
* PURPOSE
* Class to hold data for a single RTCP report header.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.io.*;

public class RtcpReportHeader {

  // RTCP Report Header Data:

  private String version;
  private String pad; 
  private int    report_count =0; // for packets 200 and 201 only
  private int    source_count =0; // for packets 202 and 203 only
  private int    subtype      =0; // for packets 204 only
  private int    packet_type  =0;
  private int    packet_len   =0;
  private long   ssrc         =0;

  public RtcpReportHeader(String version, String pad, int count_or_subtype,
                          int packet_type, int packet_len, long ssrc) {

    this.version      = version;
    this.pad          = pad; 
    this.packet_type  = packet_type;
    this.packet_len   = packet_len;
    this.ssrc         = ssrc;

    switch(packet_type) {
    case 200:
    case 201:
      this.report_count = count_or_subtype; 
      break;
    case 202:
    case 203:
      this.source_count = count_or_subtype; 
      break;
    case 204:
      this.subtype = count_or_subtype; 
      break;
    default:
      break;
    }
  }

  public RtcpReportHeader(RtcpReportHeader copy) {

    this.version      = copy.getVersion();
    this.pad          = copy.getPad(); 
    this.packet_type  = copy.getPacketType();
    this.packet_len   = copy.getPacketLen();
    this.ssrc         = copy.getSsrc();

    switch(packet_type) {
    case 200:
    case 201:
      this.report_count = copy.getReportCount();
      break;
    case 202:
    case 203:
      this.source_count = copy.getSourceCount(); 
      break;
    case 204:
      this.subtype = copy.getSubtype(); 
      break;
    default:
      break;
    }
  }

  public String getVersion() { return(version); }
  public String getPad() { return(pad); }
  public int getPacketType() { return(packet_type);   }
  public int getPacketLen() { return(packet_len);   }
  public long getSsrc() { return(ssrc);   }
  public int getReportCount() { return(report_count);   }
  public int getSourceCount() { return(source_count);   }
  public int getSubtype() { return(subtype);   }
}
