/****************************************************************************
* RtcpReportPanel.java
*
* PURPOSE
* Panel to display RTCP Report data.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;

public class RtcpReportPanel extends BorderedPanel {

  private TextField name = null;
  private TextField ip = null;
  private TextField ssrc = null;
  private TextField update = null;
  private TextField src_ssrc = null;
  private TextField fraction = null;
  private TextField cum_pack = null;
  private TextField seq_no_cycles  = null;
  private TextField high_seq_no = null;
  private TextField jitter = null;
  private TextField last_sr = null;
  private TextField delay_since_lsr = null;

  public RtcpReportPanel(RtcpReport report) {

    Label lblHeader;
    if (report.getPacketType() == 200) 
      lblHeader = new Label("RTCP Sender Report Header", Label.CENTER);
    else
      lblHeader = new Label("RTCP Receiver Report Header", Label.CENTER);
    lblHeader.setFont(new Font("Dialog", Font.BOLD, 18));
    addToGridBag(lblHeader, 0, 0, 2, 1);

    addToGridBag(new Label("Name:",Label.RIGHT), 0, 1, 1, 1);
    name = new TextField(report.getName());
    name.setEditable(false);
    addToGridBag(name, 1, 1, 1, 1);

    addToGridBag(new Label("Address:",Label.RIGHT), 0, 2, 1, 1);
    ip = new TextField(report.getIP() + "/" + report.getPort());
    ip.setEditable(false);
    addToGridBag(ip, 1, 2, 1, 1);

    addToGridBag(new Label("Ssrc:",Label.RIGHT), 0, 3, 1, 1);
    ssrc = new TextField(report.getSsrc() + "");
    ssrc.setEditable(false);
    addToGridBag(ssrc, 1, 3, 1, 1);

    addToGridBag(new Label("Timestamp:",Label.RIGHT), 0, 4, 1, 1);
    update = new TextField(report.getLastUpdate().toString());
    update.setEditable(false);
    addToGridBag(update, 1, 4, 1, 1);
  }

  public RtcpReportPanel(RtcpReport report, int reportBlock) {

    Label lblHeader = new Label("RTCP Report Block " + (reportBlock+1) +
                                " of " + report.getReportCount(), Label.CENTER);
    lblHeader.setFont(new Font("Dialog", Font.BOLD, 18));
    addToGridBag(lblHeader, 0, 0, 2, 1);

    addToGridBag(new Label("Source Ssrc:",Label.RIGHT), 0, 1, 1, 1);
    src_ssrc = new TextField(report.report_blocks[reportBlock].source_ssrc +"");
    src_ssrc.setEditable(false);
    addToGridBag(src_ssrc, 1, 1, 1, 1);

    addToGridBag(new Label("Fraction Lost:",Label.RIGHT), 0, 2, 1, 1);
    fraction = new TextField(report.report_blocks[reportBlock].fraction_lost + 
                "/256 ("+ (int)
                ((report.report_blocks[reportBlock].fraction_lost/256.0)*100) 
                 + "%)");
    fraction.setEditable(false);
    addToGridBag(fraction, 1, 2, 1, 1);
     
    addToGridBag(new Label("Cumulative Packets Lost:",Label.RIGHT), 
                 0, 3, 1, 1);
    cum_pack = new TextField(
                 report.report_blocks[reportBlock].cum_packets_lost + "");
    cum_pack.setEditable(false);
    addToGridBag(cum_pack, 1, 3, 1, 1);

    addToGridBag(new Label("Sequence Number Cycles:",Label.RIGHT), 0, 4, 1, 1);
    seq_no_cycles  = new TextField
                         (report.report_blocks[reportBlock].seq_no_cycles + "");
    seq_no_cycles.setEditable(false);
    addToGridBag(seq_no_cycles, 1, 4, 1, 1);

    addToGridBag(new Label("Highest Sequence Number:",Label.RIGHT), 0, 5, 1, 1);
    high_seq_no = new TextField
                        (report.report_blocks[reportBlock].highest_seq_no + "");
    high_seq_no.setEditable(false);
    addToGridBag(high_seq_no, 1, 5, 1, 1);

    addToGridBag(new Label("Interarrival Jitter:",Label.RIGHT), 0, 6, 1, 1);
    jitter = new TextField
                   (report.report_blocks[reportBlock].interarrival_jitter + "");
    jitter.setEditable(false);
    addToGridBag(jitter, 1, 6, 1, 1);

    addToGridBag(new Label("Last SR:",Label.RIGHT), 0, 7, 1, 1);
    last_sr = new TextField(report.report_blocks[reportBlock].last_sr + "");
    last_sr.setEditable(false);
    addToGridBag(last_sr, 1, 7, 1, 1);

    addToGridBag(new Label("Delay Since Last SR:",Label.RIGHT), 0, 8, 1, 1);
    delay_since_lsr = new TextField
                       (report.report_blocks[reportBlock].delay_since_lsr + "");
    delay_since_lsr.setEditable(false);
    addToGridBag(delay_since_lsr, 1, 8, 1, 1);
  }

  public void updateReport(RtcpReport report) {
    name.setText(report.getName());
    ip.setText(report.getIP() + "/" + report.getPort());
    ssrc.setText(report.getSsrc() + "");
    update.setText(report.getLastUpdate().toString());
  }

  public void updateReport(RtcpReport report, int reportBlock) {

    src_ssrc.setText(report.report_blocks[reportBlock].source_ssrc + "");

    fraction.setText(report.report_blocks[reportBlock].fraction_lost + 
                "/256 ("+ (int)
                ((report.report_blocks[reportBlock].fraction_lost/256.0)*100) 
                 + "%)");
     
    cum_pack.setText(report.report_blocks[reportBlock].cum_packets_lost + "");

    seq_no_cycles.setText(report.report_blocks[reportBlock].seq_no_cycles + "");

    high_seq_no.setText(report.report_blocks[reportBlock].highest_seq_no + "");
    
    jitter.setText(report.report_blocks[reportBlock].interarrival_jitter + "");

    last_sr.setText(report.report_blocks[reportBlock].last_sr + "");

    delay_since_lsr.setText(report.report_blocks[reportBlock].delay_since_lsr + "");

  }

  public void emptyReport() {
    src_ssrc.setText("");
    fraction.setText("");
    cum_pack.setText("");
    seq_no_cycles.setText("");
    high_seq_no.setText("");
    jitter.setText("");
    last_sr.setText("");
    delay_since_lsr.setText("");
  }
}
