/****************************************************************************
* RtcpSdesPanel.java
*
* PURPOSE
* Panel to display an RTCP SDES packet.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;

public class RtcpSdesPanel extends BorderedPanel {

  private TextField cname = null;
  private TextField name = null;
  private TextField email = null;
  private TextField phone = null;
  private TextField location = null;
  private TextField tool = null;
  private TextField note = null;

  private int offset=0;

  public RtcpSdesPanel(RtcpReport report) {

//    setFill("HORIZONTAL");
//    setAnchor("NORTH");
//    setWeight(0,0);

    Label lblHeader = new Label("RTCP Session Description", Label.CENTER);
    lblHeader.setFont(new Font("Dialog", Font.BOLD, 18));
    addToGridBag(lblHeader, 0, offset++, 2, 1);

    if (report.getSdesCname() != null) {
      addToGridBag(new Label("Cname:",Label.RIGHT), 0, offset, 1, 1);
      cname = new TextField(report.getSdesCname());
      cname.setEditable(false);
      addToGridBag(cname, 1, offset++, 1, 1);
    }

    if (report.getSdesName() != null) {
      addToGridBag(new Label("Name:",Label.RIGHT), 0, offset, 1, 1);
      name = new TextField(report.getSdesName());
      name.setEditable(false);
      addToGridBag(name, 1, offset++, 1, 1);
    }

    if (report.getSdesEmail() != null) {
      addToGridBag(new Label("Email:",Label.RIGHT), 0, offset, 1, 1);
      email = new TextField(report.getSdesEmail());
      email.setEditable(false);
      addToGridBag(email, 1, offset++, 1, 1);
    }

    if (report.getSdesPhone() != null) {
      addToGridBag(new Label("Phone:",Label.RIGHT), 0, offset, 1, 1);
      phone = new TextField(report.getSdesPhone());
      phone.setEditable(false);
      addToGridBag(phone, 1, offset++, 1, 1);
    }

    if (report.getSdesLoc() != null) {
      addToGridBag(new Label("Location:",Label.RIGHT), 0, offset, 1, 1);
      location = new TextField(report.getSdesLoc());
      location.setEditable(false);
      addToGridBag(location, 1, offset++, 1, 1);
    }

    if (report.getSdesTool() != null) {
      addToGridBag(new Label("Tool:",Label.RIGHT), 0, offset, 1, 1);
      tool = new TextField(report.getSdesTool());
      tool.setEditable(false);
      addToGridBag(tool, 1, offset++, 1, 1);
    }

    if (report.getSdesNote() != null) {
      addToGridBag(new Label("Note:",Label.RIGHT), 0, offset, 1, 1);
      note = new TextField(report.getSdesNote());
      note.setEditable(false);
      addToGridBag(note, 1, offset++, 1, 1);
    }
  }

  public boolean updateSdes(RtcpReport report) {

    boolean redraw = false;

    if (report.getSdesCname() != null) {
      if (cname == null) {
        addToGridBag(new Label("Cname:",Label.RIGHT), 0, offset, 1, 1);
        cname = new TextField(report.getSdesCname());
        cname.setEditable(false);
        addToGridBag(cname, 1, offset++, 1, 1);
        cname.invalidate();
        this.invalidate();
        redraw = true;
      } else 
        cname.setText(report.getSdesCname());
    }

    if (report.getSdesName() != null) {
      if (name == null) {
        addToGridBag(new Label("Name:",Label.RIGHT), 0, offset, 1, 1);
        name = new TextField(report.getSdesName());
        name.setEditable(false);
        addToGridBag(name, 1, offset++, 1, 1);
        name.invalidate();
        this.invalidate();
        redraw = true;
      } else
        name.setText(report.getSdesName());
    }

    if (report.getSdesEmail() != null) {
      if (email == null) {
        addToGridBag(new Label("Email:",Label.RIGHT), 0, offset, 1, 1);
        email = new TextField(report.getSdesEmail());
        email.setEditable(false);
        addToGridBag(email, 1, offset++, 1, 1);
        email.invalidate();
        this.invalidate();
        redraw = true;
      } else 
        email.setText(report.getSdesEmail());
    }

    if (report.getSdesPhone() != null) {
      if (phone == null) {
        addToGridBag(new Label("Phone:",Label.RIGHT), 0, offset, 1, 1);
        phone = new TextField(report.getSdesPhone());
        phone.setEditable(false);
        addToGridBag(phone, 1, offset++, 1, 1);
        phone.invalidate();
        this.invalidate();
        redraw = true;
      } else 
        phone.setText(report.getSdesPhone());
    }

    if (report.getSdesLoc() != null) {
      if (location == null) {
        addToGridBag(new Label("Location:",Label.RIGHT), 0, offset, 1, 1);
        location = new TextField(report.getSdesLoc());
        location.setEditable(false);
        addToGridBag(location, 1, offset++, 1, 1);
        location.invalidate();
        this.invalidate();
        redraw = true;
      } else 
        location.setText(report.getSdesLoc());
    }

    if (report.getSdesTool() != null) {
      if (tool == null) {
        addToGridBag(new Label("Tool:",Label.RIGHT), 0, offset, 1, 1);
        tool = new TextField(report.getSdesTool());
        tool.setEditable(false);
        addToGridBag(tool, 1, offset++, 1, 1);
        tool.invalidate();
        this.invalidate();
        redraw = true;
      } else 
        tool.setText(report.getSdesTool());
    }

    if (report.getSdesNote() != null) {
      if (note == null) {
        addToGridBag(new Label("Note:",Label.RIGHT), 0, offset, 1, 1);
        note = new TextField(report.getSdesNote());
        note.setEditable(false);
        addToGridBag(note, 1, offset++, 1, 1);
        note.invalidate();
        this.invalidate();
        redraw = true;
      } else
        note.setText(report.getSdesNote());
    }
    return(redraw);
  }
}
