/****************************************************************************
* ScrollableTreePanel.java
*
* PURPOSE
* Panel to display an MHealth tree.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public
class ScrollableTreePanel extends Panel implements ScrollInterface,
                                                   AdjustmentListener {
  Scrollbar vscroller = null; 
  Scrollbar hscroller = null; 
  Dimension dimensions;
  private SessionData sessionData;
  public TreePanel treepanel; 
  
  public ScrollableTreePanel(SessionData sessionData, int width,int height) {
    this.sessionData = sessionData;

    dimensions = new Dimension(width,height);
    setLayout(new BorderLayout());

    treepanel = new TreePanel(this, sessionData, width+10, height);

    //treepanel.startListening();
    add("Center", treepanel);
   
    vscroller=new Scrollbar(Scrollbar.VERTICAL);
    add("East", vscroller);
 
    hscroller=new Scrollbar(Scrollbar.HORIZONTAL);
    add("South", hscroller);
   
    vscroller.addAdjustmentListener(this);
    hscroller.addAdjustmentListener(this);
  }

  public int getVerticalScrollerVal() {
    return(vscroller.getValue());
  }
  
  public int getVerticalScrollerVis() {
    return(vscroller.getVisibleAmount());
  }
  
  public int getVerticalScrollerMin() {
    return(vscroller.getMinimum());
  }
  
  public int getVerticalScrollerMax() {
    return(vscroller.getMaximum());
  }
  
  public void setVerticalScrollerValues(int val,int visible,int min,int max) {
    vscroller.setValues(val,visible,min,max);
  }
  
  public int getHorizontalScrollerVal() {
    return(hscroller.getValue());
  }
  
  public int getHorizontalScrollerVis() {
    return(hscroller.getVisibleAmount());
  }
  
  public int getHorizontalScrollerMin() {
    return(hscroller.getMinimum());
  }
  
  public int getHorizontalScrollerMax() {
    return(hscroller.getMaximum());
  }
  
  public void setHorizontalScrollerValues(int val,int visible,int min,int max) {
    hscroller.setValues(val,visible,min,max);
  }
  
  // AdjustmentListener interface adjustmentValueChanged():
  public void adjustmentValueChanged(AdjustmentEvent evt) {
    treepanel.repaint();
  }
  
  public Dimension getPreferredSize() {
//    System.out.println("ScrollableTreePanel::getPreferredSize() is " +
//                       dimensions);
    
    return(dimensions);
  }

  public void redraw() {
    treepanel.redraw();
  }

  public void createImageDump() {
    treepanel.createImageDump();
  }
  
}
