/****************************************************************************
* SessionData.java
*
* PURPOSE
* Class to store session data for an MHealth session (mc address, ports, 
* etc).
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.util.*;
import java.io.*;

public class SessionData {

  private String SessionIP="";
  private String SessionPort="";

  public final static int STOPPED = 0;
  public final static int STARTED = 1;
  public int status = STOPPED;

  private boolean RtcpLoggingEnabled=false;
  private boolean RouteLoggingEnabled=false;

  private String LogDir="/tmp/";
  private String LogFileName="mhealth";

  private boolean ImageDumpEnabled=false;
  private String ImageDumpDir="/tmp/";
  private String ImageDumpFreq="30";

  private String SessionAddr="";
  private String LocalTime="";
  private String SourceCount="0";
  private String ReceiverCount="0";
  private String StatusText="";

  public ReceiverList receiverList;
  public SourceList sourceList;

  public Tree tree;

  public FileWriter RouteLog=null;

  public SessionData() {
    receiverList = new ReceiverList();
    sourceList = new SourceList();
    tree = new Tree();
  }  

  public SessionData(String SessionIP, String SessionPort, 
                     boolean RtcpLoggingEnabled, boolean RouteLoggingEnabled, 
                     String LogDir) {
    this.SessionIP = SessionIP;
    this.SessionPort = SessionPort;
  
    this.RtcpLoggingEnabled = RtcpLoggingEnabled;
    this.RouteLoggingEnabled = RouteLoggingEnabled;

    this.LogDir = LogDir;

    receiverList = new ReceiverList();
    sourceList = new SourceList();
  }

  public String getSessionIP() { return(SessionIP); }
  public boolean setSessionIP(String SessionIP) { 
    this.SessionIP = SessionIP; 
    return(true);
  }

  public String getSessionPort() { return(SessionPort); }
  public boolean setSessionPort(String SessionPort) {
    this.SessionPort = SessionPort; 
    return(true);
  }

  public boolean getRtcpLoggingEnabled() { return(RtcpLoggingEnabled); }
  public void setRtcpLoggingEnabled(boolean RtcpLoggingEnabled) {
    this.RtcpLoggingEnabled = RtcpLoggingEnabled; 
  }

  public boolean getRouteLoggingEnabled() { return(RouteLoggingEnabled); }
  public void setRouteLoggingEnabled(boolean RouteLoggingEnabled) {
    this.RouteLoggingEnabled = RouteLoggingEnabled; 
  }

  public String getLogDir() { return(LogDir); }
  public boolean setLogDir(String LogDir) {
    this.LogDir = LogDir; 
    return(true);
  }

  public String getLogFileName() { return(LogFileName); }
  public void setLogFileName(String LogFileName) { 
    this.LogFileName = LogFileName; 
  }

  public boolean getImageDumpEnabled() { return(ImageDumpEnabled); }
  public void setImageDumpEnabled(boolean ImageDumpEnabled) { 
    this.ImageDumpEnabled = ImageDumpEnabled; 
  }

  public String getImageDumpDir() { return(ImageDumpDir); }
  public void setImageDumpDir(String ImageDumpDir) { 
    this.ImageDumpDir = ImageDumpDir; 
  }

  public String getImageDumpFreq() { return(ImageDumpFreq); }
  public void setImageDumpFreq(String ImageDumpFreq) { 
    this.ImageDumpFreq = ImageDumpFreq; 
  }

  public String getSessionAddr() { return(SessionAddr); }
  public void setSessionAddr(String SessionAddr) { 
    this.SessionAddr = SessionAddr; 
  }

  public String getLocalTime() { return(LocalTime); }
  public void setLocalTime(String LocalTime) { 
    this.LocalTime = LocalTime; 
  }

  public String getSourceCount() { return(SourceCount); }
  public void setSourceCount(String SourceCount) { 
    this.SourceCount = SourceCount; 
  }

  public String getReceiverCount() { return(ReceiverCount); }
  public void setReceiverCount(String ReceiverCount) { 
    this.ReceiverCount = ReceiverCount; 
  }

  public String getStatusText() { return(StatusText); }
  public void setStatusText(String StatusText) { 
    this.StatusText = StatusText; 
  }

}
