/****************************************************************************
* StatsWin.java
*
* PURPOSE
* Window to provide MHealth stats.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.lang.*;

public class StatsWin extends Frame implements RtcpLoggerListener {

  private Button btnDismiss;
  
  private String buttonPressed;

  private String IP;

  private RtcpLogger rtcplogger;
  private TreeNode treenode;
  private RtcpReport report;
  private int xLoc, yLoc;

  private RtcpReportPanel rtcpHeader;
  private RtcpReportPanel rtcpBlocks[];
  private int reportCount=0;
  private RtcpSdesPanel rtcpSdes;
  private Panel pnlReports;

  public StatsWin(TreeNode treenode, RtcpReport report, RtcpLogger rtcplogger,
                  int xLoc, int yLoc) {
    super("Mhealth Node Stats");

    if (treenode != null) IP = treenode.IP;
    else if (report != null) IP = report.getIP();

    this.rtcplogger = rtcplogger;
    this.treenode = treenode;
    this.report = report;
    this.xLoc = xLoc;
    this.yLoc = yLoc;
  
    drawStatsWin();
  }

  public void drawStatsWin() {

    int maxWidth = 0;
    int reportWidth = 0;

    setSize(400,300);
    setLocation(xLoc,yLoc);
    addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent event) {
                          setVisible(false);
                          dispose();
                        }
                      });


    GridBagLayout layout = new GridBagLayout();
    setLayout(layout);

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 0;
    gbc.weighty = 0;

    int xOffset = 0;
    int yOffset = 0;

    if (report != null) {   
      rtcpHeader = new RtcpReportPanel(report);
      addToGridBag(rtcpHeader, gbc, xOffset++, yOffset, 1, 1);
      rtcpSdes = new RtcpSdesPanel(report);
      addToGridBag(rtcpSdes, gbc, xOffset++, yOffset, 1, 1);
      if (treenode != null) {
        int xM,yM;
        switch (report.getReportCount()) {
        case 0:
          xM = xOffset++;
          yM = yOffset++;
          maxWidth = 3;
          break;
        case 1:
          xM = xOffset-1;
          yM = ++yOffset;
          maxWidth = 2;
          reportWidth = 1;
          break;
        default:
          xM = xOffset++;
          yM = yOffset++;
          maxWidth = reportWidth = 3;
          break;
        }
        addToGridBag(new MtraceStatsPanel(treenode.mtraceStats, null, null), 
                     gbc, xM, yM, 1, 1);
      } else {
        yOffset++;
        reportWidth = maxWidth = 2;
      }

      reportCount = report.getReportCount();
      if (reportCount > 0) { 
        int reportHeight = (int) Math.ceil(reportCount/3.0);

        pnlReports = new Panel(new GridLayout(reportHeight, reportWidth, 0, 0));

        rtcpBlocks = new RtcpReportPanel[reportCount];
        for (int x=0; x < reportCount; x++) {
          rtcpBlocks[x] = new RtcpReportPanel(report,x);
          pnlReports.add(rtcpBlocks[x]);
        }

/*        if (reportCount > 4) 
          maxWidth = 3;
        else 
          maxWidth = 2;*/

        xOffset=0;
        addToGridBag(pnlReports, gbc, xOffset, yOffset, reportWidth,reportHeight);
        yOffset += reportHeight;
      }
    } else if (treenode != null) {
      addToGridBag(new MtraceStatsPanel(treenode.mtraceStats, treenode.name,
                                        treenode.IP),  
                   gbc, xOffset, yOffset++, 1, 1);
      maxWidth = 1;
    }

//    System.out.println("yOffset = " + yOffset);
    btnDismiss = new Button("Dismiss");
    btnDismiss.addActionListener(
               new ActionListener() {
                 public void actionPerformed(ActionEvent evt) {
                   closeWin();
                 } 
               });

    addToGridBag(btnDismiss, gbc, 0, yOffset, maxWidth, 1);

    addWindowListener(new WindowAdapter() { 
                        public void windowClosing(WindowEvent e) { 
                          closeWin();
                        } 
                      } );
  
    pack();
    Dimension s = getSize();
//    System.out.println("size is " + s);
//    setResizable(false);

    rtcplogger.addRtcpLoggerListener(this);

    setVisible(true);

  }

  public void closeWin() {
    rtcplogger.removeRtcpLoggerListener(this);
    setVisible(false);
    dispose();
  }

  public void addToGridBag(Component c, GridBagConstraints gbc,
                           int x, int y, int w, int h) {
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    add(c, gbc);
  }

  public void senderAdded(RtcpReport report) {}
  public void senderRemoved(RtcpReport report) {}

  public void senderUpdated(RtcpReport report, boolean redraw) {
    if (IP.equals(report.getIP()))
      updateWin(report);
  }

  public void receiverAdded(RtcpReport report) {}
  public void receiverRemoved(RtcpReport report) {} 

  public void receiverUpdated(RtcpReport report, boolean redraw) {
    if (IP.equals(report.getIP()))
      updateWin(report);
  }

  public void receiverTimeout(RtcpReport report) {}

  public void updateWin(RtcpReport report) {

    this.report = report;

    if (rtcpHeader != null) 
      rtcpHeader.updateReport(report);

    if (rtcpSdes != null) {
      if (rtcpSdes.updateSdes(report))
        this.validate();
    }

    if (report.getReportCount() == reportCount) {
      for (int x=0; x < reportCount; x++) {
        rtcpBlocks[x].updateReport(report,x);
      }
    } else if (report.getReportCount() < reportCount) {
      for (int x=0; x < reportCount; x++) {
        if (x >= report.getReportCount())
          rtcpBlocks[x].emptyReport();
        else
          rtcpBlocks[x].updateReport(report,x);
      }
    } else if (report.getReportCount() > reportCount) {
      drawStatsWin();
    }
  }
}
